@@ -95,6 +95,8 @@ struct _PxManager {
   gboolean wpad;
   GBytes *pac_data;
   char *pac_url;
+
+  GMutex mutex;
 };
 
 G_DEFINE_TYPE (PxManager, px_manager, G_TYPE_OBJECT)
@@ -390,15 +392,37 @@ px_manager_pac_download (PxManager  *self,
   if (g_str_has_prefix (url, "pac+"))
     url += 4;
 
-  curl_easy_setopt (self->curl, CURLOPT_NOSIGNAL, 1);
-  curl_easy_setopt (self->curl, CURLOPT_FOLLOWLOCATION, 1);
-  curl_easy_setopt (self->curl, CURLOPT_NOPROXY, "*");
-  curl_easy_setopt (self->curl, CURLOPT_CONNECTTIMEOUT, 30);
-  curl_easy_setopt (self->curl, CURLOPT_USERAGENT, "libproxy");
+  if (curl_easy_setopt (self->curl, CURLOPT_NOSIGNAL, 1) != CURLE_OK)
+    g_debug ("Could not set NOSIGNAL, continue");
 
-  curl_easy_setopt (self->curl, CURLOPT_URL, url);
-  curl_easy_setopt (self->curl, CURLOPT_WRITEFUNCTION, store_data);
-  curl_easy_setopt (self->curl, CURLOPT_WRITEDATA, byte_array);
+  if (curl_easy_setopt (self->curl, CURLOPT_FOLLOWLOCATION, 1) != CURLE_OK)
+    g_debug ("Could not set FOLLOWLOCATION, continue");
+
+  if (curl_easy_setopt (self->curl, CURLOPT_NOPROXY, "*") != CURLE_OK) {
+    g_warning ("Could not set NOPROXY, ABORT!");
+    return NULL;
+  }
+
+  if (curl_easy_setopt (self->curl, CURLOPT_CONNECTTIMEOUT, 30) != CURLE_OK)
+    g_debug ("Could not set CONENCTIONTIMEOUT, continue");
+
+  if (curl_easy_setopt (self->curl, CURLOPT_USERAGENT, "libproxy") != CURLE_OK)
+    g_debug ("Could not set USERAGENT, continue");
+
+  if (curl_easy_setopt (self->curl, CURLOPT_URL, url) != CURLE_OK) {
+    g_warning ("Could not set URL, ABORT!");
+    return NULL;
+  }
+
+  if (curl_easy_setopt (self->curl, CURLOPT_WRITEFUNCTION, store_data) != CURLE_OK) {
+    g_warning ("Could not set WRITEFUNCTION, ABORT!");
+    return NULL;
+  }
+
+  if (curl_easy_setopt (self->curl, CURLOPT_WRITEDATA, byte_array) != CURLE_OK) {
+    g_warning ("Could not set WRITEDATA, ABORT!");
+    return NULL;
+  }
 
   res = curl_easy_perform (self->curl);
   if (res != CURLE_OK) {
@@ -449,9 +473,6 @@ px_manager_run_pac (PxPacRunner  *pacrunner,
   g_auto (GStrv) proxies_split = NULL;
   char *pac_response;
 
-  if (!ifc->set_pac (PX_PAC_RUNNER (pacrunner), pac))
-    return;
-
   pac_response = ifc->run (PX_PAC_RUNNER (pacrunner), uri);
 
   /* Split line to handle multiple proxies */
@@ -529,6 +550,22 @@ px_manager_expand_wpad (PxManager *self,
   return ret;
 }
 
+static gboolean
+px_manager_set_pac (PxManager *self)
+{
+  GList *list;
+
+  for (list = self->pacrunner_plugins; list && list->data; list = list->next) {
+    PxPacRunner *pacrunner = PX_PAC_RUNNER (list->data);
+    PxPacRunnerInterface *ifc = PX_PAC_RUNNER_GET_IFACE (pacrunner);
+
+    if (!ifc->set_pac (PX_PAC_RUNNER (pacrunner), self->pac_data))
+      return FALSE;
+  }
+
+  return TRUE;
+}
+
 static gboolean
 px_manager_expand_pac (PxManager *self,
                        GUri      *uri)
@@ -561,6 +598,12 @@ px_manager_expand_pac (PxManager *self,
         ret = FALSE;
       } else {
         g_debug ("%s: PAC recevied!", __FUNCTION__);
+        if (!px_manager_set_pac (self)) {
+          g_warning ("%s: Unable to set PAC from %s while online = %d!", __FUNCTION__, self->pac_url, self->online);
+          g_clear_pointer (&self->pac_url, g_free);
+          g_clear_pointer (&self->pac_data, g_bytes_unref);
+          ret = FALSE;
+        }
       }
     }
   }
@@ -582,13 +625,19 @@ px_manager_get_proxies_sync (PxManager   *self,
                              const char  *url,
                              GError     **error)
 {
-  g_autoptr (GStrvBuilder) builder = g_strv_builder_new ();
-  g_autoptr (GUri) uri = g_uri_parse (url, G_URI_FLAGS_PARSE_RELAXED, error);
+  g_autoptr (GStrvBuilder) builder = NULL;
+  g_autoptr (GUri) uri = NULL;
   g_auto (GStrv) config = NULL;
 
+  g_mutex_lock (&self->mutex);
+
+  builder = g_strv_builder_new ();
+  uri = g_uri_parse (url, G_URI_FLAGS_PARSE_RELAXED, error);
+
   g_debug ("%s: url=%s online=%d", __FUNCTION__, url ? url : "?", self->online);
   if (!uri || !self->online) {
     px_strv_builder_add_proxy (builder, "direct://");
+    g_mutex_unlock (&self->mutex);
     return g_strv_builder_end (builder);
   }
 
@@ -619,6 +668,7 @@ px_manager_get_proxies_sync (PxManager   *self,
   for (int idx = 0; idx < ((GPtrArray *)builder)->len; idx++)
     g_debug ("%s: Proxy[%d] = %s", __FUNCTION__, idx, (char *)((GPtrArray *)builder)->pdata[idx]);
 
+  g_mutex_unlock (&self->mutex);
   return g_strv_builder_end (builder);
 }
 
@@ -696,8 +746,8 @@ static gboolean
 ignore_ip (GUri *uri,
            char *ignore)
 {
-  GInetAddress *uri_address;
-  GInetAddress *ignore_address;
+  g_autoptr (GInetAddress) uri_address = NULL;
+  g_autoptr (GInetAddress) ignore_address = NULL;
   g_auto (GStrv) ignore_split = NULL;
   g_autoptr (GError) error = NULL;
   const char *uri_host = g_uri_get_host (uri);
